/*
 * Copyright (c) 2012 NINTENDO.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions, and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY NINTENDO AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL NINTENDO OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef MEDIAPLAYERPRIVATEWIIU_H
#define MEDIAPLAYERPRIVATEWIIU_H

class MEDIApluginPlayer;
class MEDIAdecodedVideoFrame;
class MediaInterfaceBridge;
class MEDIAtransportFileHook;



#ifndef WK_MEDIAPLAYER_RESOURCEHOOKS_ONLY

#if ENABLE(VIDEO) && PLATFORM(WIIU)

#include "../MediaPlayerPrivate.h"
#include "Timer.h"
#include <wkgfx_MP_Core.h>

//! Class to derive timer callbacks from using the internal timer functions in MediaPlayerPrivateWiiU (internal use only)
class MediaTimer
{
public:
	void _timerCallback(WebCore::Timer<MediaTimer>*);
	virtual void timerCallback(void) = 0;
};


namespace WebCore {

class GraphicsContext;
class IntSize;
class IntRect;

class MediaPlayerPrivateWiiU : public MediaInterfaceRenderingPlayerBase, public MediaPlayerPrivateInterface
#if USE(ACCELERATED_COMPOSITING)
	, public GraphicsLayerClient
#endif
	{
public:
	static void registerMediaEngine(MediaEngineRegistrar);

	~MediaPlayerPrivateWiiU();

    void load(const String& url);
    void cancelLoad();

    void prepareToPlay();
#if USE(ACCELERATED_COMPOSITING)
    PlatformLayer* platformLayer() const;
#endif

    void play();
    void pause();

	bool isEnded();
	void timeChanged();
	bool inMediaDocument() { return m_player && m_player->inMediaDocument(); }

    virtual bool supportsFullscreen() const { return false; }
    virtual bool supportsSave() const { return false; }

    IntSize naturalSize() const;

    bool hasVideo() const;
    bool hasAudio() const;

    void setVisible(bool);

    float duration() const;

    float currentTime() const;
    void seek(float time);
    bool seeking() const;

    virtual float startTime() const { return 0; }

    virtual double initialTime() const { return 0; }

    void setRate(float);
    virtual void setPreservesPitch(bool) { }

    bool paused() const;

    void setVolume(float);
	virtual int getDefaultDeviceId(void);
	virtual void setDeviceVolume(float, int);
	virtual bool isFinished(void);
	virtual bool canRestart(void);
	virtual void restart(void);

    virtual bool supportsMuting() const { return true; }
    virtual void setMuted(bool);
	virtual void setLoop(bool loop);

    virtual bool hasClosedCaptions() const { return false; }
    virtual void setClosedCaptionsVisible(bool) { }

    MediaPlayer::NetworkState networkState() const;
    MediaPlayer::ReadyState readyState() const;

    float maxTimeSeekable() const;
    PassRefPtr<TimeRanges> buffered() const;

    unsigned bytesLoaded() const;

    void setSize(const IntSize&);

    void paint(GraphicsContext*, const IntRect&);

    virtual void paintCurrentFrameInContext(GraphicsContext* c, const IntRect& r) { paint(c, r); }

    void setPreload(MediaPlayer::Preload);

    virtual bool hasAvailableVideoFrame() const { return readyState() >= MediaPlayer::HaveCurrentData; }

    virtual bool canLoadPoster() const { return false; }
    virtual void setPoster(const String&) { }

#if ENABLE(PLUGIN_PROXY_FOR_VIDEO)
    virtual void deliverNotification(MediaPlayerProxyNotificationType) = 0;
    virtual void setMediaPlayerProxy(WebMediaPlayerProxy*) = 0;
    virtual void setControls(bool) { }
    virtual void enterFullscreen() { }
    virtual void exitFullscreen() { }
#endif

#if USE(ACCELERATED_COMPOSITING)
    // whether accelerated rendering is supported by the media engine for the current media.
    virtual bool supportsAcceleratedRendering() const;

    // called when the rendering system flips the into or out of accelerated rendering mode.
    virtual void acceleratedRenderingStateChanged();

    // Callback for when hardware-accelerated animation started.
    virtual void notifyAnimationStarted(const GraphicsLayer*, double time);

    // Notification that a layer property changed that requires a subsequent call to syncCompositingState()
    // to appear on the screen.
    virtual void notifySyncRequired(const GraphicsLayer*);

    virtual void paintContents(const GraphicsLayer*, GraphicsContext&, GraphicsLayerPaintingPhase, const IntRect& inClip);
    virtual void didCommitChangesForLayer(const GraphicsLayer*) const;

    virtual bool showDebugBorders() const;
    virtual bool showRepaintCounter() const;
#endif

    virtual bool hasSingleSecurityOrigin() const;

    virtual MediaPlayer::MovieLoadType movieLoadType() const { return MediaPlayer::Unknown; }

    virtual void prepareForRendering() { }

    // Time value in the movie's time scale. It is only necessary to override this if the media
    // engine uses rational numbers to represent media time.
    virtual float mediaTimeForTimeValue(float timeValue) const { return timeValue; }

    // Overide this if it is safe for HTMLMediaElement to cache movie time and report
    // 'currentTime' as [cached time + elapsed wall time]. Returns the maximum wall time
    // it is OK to calculate movie time before refreshing the cached time.
    virtual double maximumDurationToCacheMediaTime() const { return 0; }

    virtual unsigned decodedFrameCount() const { return 0; }
    virtual unsigned droppedFrameCount() const { return 0; }
    virtual unsigned audioDecodedByteCount() const { return 0; }
    virtual unsigned videoDecodedByteCount() const { return 0; }

    void getSitesInMediaCache(Vector<String>&) { }
    void clearMediaCache() { }
    void clearMediaCacheForSite(const String&) { }

    virtual void setPrivateBrowsingMode(bool) { }


#if ENABLE(WEB_AUDIO)
    virtual AudioSourceProvider* audioSourceProvider() { return 0; }
#endif

#if ENABLE(MEDIA_SOURCE)
    virtual bool sourceAppend(const unsigned char*, unsigned) { return false; }
    virtual void sourceEndOfStream(MediaPlayer::EndOfStreamStatus) { };
#endif

	//
	// Platform Specific API outside standard interface definition
	//

	bool newReadyState(MediaPlayer::ReadyState newState);
	bool newNetworkState(MediaPlayer::NetworkState newState);

	void updateMetaData();
	bool haveEnoughData(MEDIApluginPlayer *pPlayer);

	virtual void setMetaData(const MediaBridge::MetaData & metaData);
	virtual void useImage(const MEDIAdecodedVideoFrame *pCurrentRenderImage);

	void setErrorState();

	void repaint();

	// - - - - -  internal use only - - - - -

	int getDisplayIndex() const;

	static WebCore::Timer<MediaTimer>* CreateTimer(MediaTimer *pTimer);
	static void DestroyTimer(WebCore::Timer<MediaTimer>* pTimer);
	static void StartTimer(WebCore::Timer<MediaTimer>* pTimer, double period);
	static void StopTimer(WebCore::Timer<MediaTimer>* pTimer);

	static bool stringEndsWith(const std::string & a, const std::string & b);
	static bool isURLallowable(const std::string& url);

private:
	struct WorkerTimer
	{
		struct PlayerInfo
		{
			PlayerInfo(MediaPlayerPrivateWiiU *pInterface) : m_pInterface(pInterface) {}

			MediaPlayerPrivateWiiU *m_pInterface;
		};
		struct MyTimer : public MediaTimer
		{
			MyTimer(void) { ownerP = NULL; }
			virtual void timerCallback(void)
			{
				if(ownerP)
					ownerP->timerCallback();
			}

			WorkerTimer *ownerP;
		} timer;

		typedef	std::vector<PlayerInfo>	PlayerInfoVector;

		pthread_mutex_t					m_playersMutex;
		PlayerInfoVector				m_players;
		int m_refCount;
		Timer<MediaTimer>* m_timerP;

		WorkerTimer(void);
		void timerCallback(void);
		void Reset(void);
		bool Setup(void);
		bool Destroy(void);
		bool RegisterPlayer(MediaPlayerPrivateWiiU *playerP);
		bool UnregisterPlayer(MediaPlayerPrivateWiiU *playerP);
	};
	static WorkerTimer s_workerTimer;

	MediaPlayerPrivateWiiU(MediaPlayer*, MEDIAtransportFileHook *hookP=NULL);
	bool isValid(void);	// if the allocation of internal stuff is ok, this returns true!

    // engine support
	static PassOwnPtr<MediaPlayerPrivateInterface> create(MediaPlayer*);
    static void getSupportedTypes(HashSet<String>& types);
    static MediaPlayer::SupportsType supportsType(const String& type, const String& codecs);
    static bool isAvailable();

	void openStream();

	void loadingFailed(MediaPlayer::NetworkState error);

	void lockDataMutex();
	void unlockDataMutex();

	friend class MediaInterfaceBridge;
	struct PeriodicHandleParam;
	void handlePeriodic(PeriodicHandleParam *pParam);

private:
	class BufferMonitor;

	enum MediaType {
		eMediaType_ADTS = 0,
		eMediaType_HLS,
		eMediaType_MP2TS,
		eMediaType_MP4,
		eMediaType_OggVorbis,
		eMediaType_MP3,
		eMediaType_WAV,

		eMediaType_Unknown,
	};

	MediaPlayer*						m_player;

	MEDIApluginPlayer *					m_pPlayer;
	BufferMonitor *						m_pPlayerBufferMonitor;

	std::string							m_url;
	std::string							m_type;
	std::string							m_codecs;

	MediaPlayer::Preload				m_preLoadMode;

#if OS(WINDOWS)
	CRITICAL_SECTION					m_dataMutex;
#else
	pthread_mutex_t						m_dataMutex;
#endif

	volatile bool						m_bMetaDataNew;
	MediaBridge::MetaData							m_metaData;

	bool								m_bVisible;
    IntSize								m_size;

    MediaPlayer::NetworkState			m_networkState;
    MediaPlayer::ReadyState				m_readyState;

	float								m_seekTime;				//!< PTS returned while a seek operation is ongoing
	float								m_resumeTime;			//!< for pause/unpause
	float   							m_mediaDuration;

	float								m_volume;
	bool								m_bMuted;
	bool								m_bLoop;

	bool								m_bPaused;				//!< True if logical state is paused (internal playback state may differ!)
	bool								m_bSeeking;
	bool								m_bEndReached;
	bool								m_bMediaDurationKnown;

	bool								m_bErrorOccurred;

	const MEDIAdecodedVideoFrame *		m_pCurrentImage;

	static String						s_lastQueryType;
	static String						s_lastQueryCodecs;


#if OS(WINDOWS)
	bool		 m_DEBUG_WasPaused;
	unsigned int m_DEBUG_PlayDuration;
	unsigned int m_DEBUG_LastPlayTime;
	unsigned int m_DEBUG_CurrentPlayTime;
#endif
	};

};


#endif
#endif

void MediaPlayer_SetupGlobalMediaResources();
void MediaPlayer_ReleaseGlobalMediaResources();
void MediaPlayer_AddPeriodicCallback(void (*callback)(float dt, void *), void *pUserData);
void MediaPlayer_RemovePeriodicCallback(void (*callback)(float dt, void *), void *pUserData);


#endif
