#ifndef CONFIG_H
#define CONFIG_H

/**
	This namespace is for holding onto settings from config.xml
*/
#include <string>
#include <vector>
#include <PlatformDefines.h>

namespace LibPlatform 
{
    struct DisplayRemapInfo
    {
        int srcX;
        int srcY;
        int srcWidth;
        int srcHeight;
        int dstX;
        int dstY;
        int dstWidth;
        int dstHeight;
    };

    enum ViewportFilterMode
    {
        POINT_FILTER,
        BILINEAR_FILTER
    };

	namespace ApplicationConfig 
	{
		struct FileHook	// pretty much the same thing in the mediaTransportFileHook
		{
		public:
			virtual long long GetFileSize(int fd) = 0;
			virtual int Open(const char *url) = 0;
			virtual int Read(int fd, long long offset, void *buffer, int count) = 0;
			virtual void Close(int fd) = 0;
		};
		void SetMediaPlayer(int mediaPlayer);
		int GetMediaPlayer();
        void SetTVDisplayDimensions(int TVDisplayWidth, int TVDisplayHeight);
        void GetTVDisplayDimensions(int &outTVDisplayWidth, int &outTVDisplayHeight);
        void SetExtendedDOMSize(int DOMWidth, int DOMHeight);
        void GetExtendedDOMSize(int &outDOMWidth, int &outDOMHeight);
        void SetExtendedDisplay(bool extendedDisplay);
        bool GetExtendedDisplay();
		void SetIsServiceApplication(bool serviceApp);
        bool GetIsServiceApplication();
		void SetNEXGameServerID(int serverID);
		int GetNEXGameServerID();
		void SetNEXAccessKey(const wchar_t * accessKey);
		std::wstring GetNEXAccessKey();
        void SetTVRemapInfo(int srcX, int srcY, int srcWidth, int srcHeight, int dstX, int dstY, int dstWidth, int dstHeight);
        void SetGP0RemapInfo(int srcX, int srcY, int srcWidth, int srcHeight, int dstX, int dstY, int dstWidth, int dstHeight);
        const DisplayRemapInfo* GetTVRemapInfo();
        const DisplayRemapInfo* GetGP0RemapInfo();
        bool IsTVRemapDirty();
        bool IsGP0RemapDirty();
        void SetTVViewportFilterMode(ViewportFilterMode filterMode);
        void SetGP0ViewportFilterMode(ViewportFilterMode filterMode);
        ViewportFilterMode GetTVViewportFilterMode();
        ViewportFilterMode GetGP0ViewportFilterMode();

		void RegisterFileHook(FileHook *hookP);
		FileHook* GetFileHook(void);
        void RegisterControllerRemapFilePath(const std::string &JSONInputRemapFilePath);
        std::vector<const std::string>* GetControllerRemapFilePathList();
        void ClearAllControllerRemapFilePaths();
        bool SetControllerRemap(const std::string *inputDevice, const std::string *deviceButton, const std::string *keyboardKey);
        signed GetControllerRemap(ControllerID ID, unsigned deviceButton);
        void ClearControllerRemaps(ControllerID ID);
		void ClearAllControllerRemaps(void);
	}
}

#endif//CONFIG_H
